/********************************************************************
 * Copyright (C) 2007 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Beesoft Tracer.
 *
 * Beesoft Tracer is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Beesoft Tracer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Beesoft Tracer
 * FILE   : BTCallArgs.cpp
 * DATE   : 27.06.2007
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include <BTCallArgs.h>
#include <iostream>
#include <vector>
#include <string>
#include <cctype>
#include <cstdlib>
using namespace std;

/*------- local constants:
-------------------------------------------------------------------*/
const int         BTCallArgs::DEFAULT_PORT = 6789;
const std::string BTCallArgs::DEFAULT_ADDR = "127.0.0.1";
const std::string BTCallArgs::ADDR_MARKER  = "-ip";
const std::string BTCallArgs::PORT_MARKER  = "-port";
const std::string BTCallArgs::HELP1        = "-h";
const std::string BTCallArgs::HELP2        = "--help";

//*******************************************************************
// BTCallArgs                                            CONSTRUCTOR
//*******************************************************************
BTCallArgs::BTCallArgs( const int in_n, const char** const in_args )
: port_ ( -1 )
, addr_ ( string() )
, help_ ( false )
{
   port_ = DEFAULT_PORT;
   addr_ = DEFAULT_ADDR;

   vector<string> params;
   params.assign( in_args + 1, in_args + in_n );

   for( int i = 0; i < params.size(); ++i ) {
      const string item = params[ i ];
      if( ( HELP1 == item ) || ( HELP2 == item ) ) {
         help_ = true;
      }
      else if( PORT_MARKER == item ) {
         ++i;
         if( i < params.size() ) {
            if( is_decnumber( params[ i ] ) ) {
               const int pt = atoi( params[ i ].c_str() );
               if( pt > 0 ) {
                  port_ = pt;
               }
            }
         }
      }
      else if( ADDR_MARKER == item ) {
         ++i;
         if( i < params.size() ) {
            if( is_ip4addr( params[ i ] ) ) {
               addr_ = params[ i ];
            }
         }
      }
   }
}
// end of BTCallArgs

//*******************************************************************
// is_ip4addr                                                PRIVATE
//*******************************************************************
bool BTCallArgs::is_ip4addr( const string& in_text ) const
{
   bool retval = false;

   vector<string> items;
   split( in_text, items );
   if( 4 == items.size() ) {
      retval = true;
      for( int i = 0; i < 4; ++i ) {
         if( !is_decnumber( items[i] ) ) {
            retval = false;
            break;
         }
      }
   }
   return retval;
}
// end of is_ip4addr

//*******************************************************************
// split                                                     PRIVATE
//*******************************************************************
void BTCallArgs::split( const string& in_text, vector<string>& out_data ) const
{
   out_data.clear();
   
   unsigned int pos1 = 0;
   unsigned int pos2 = in_text.find( '.' );

   while( pos2 != string::npos ) {
      out_data.push_back( in_text.substr( pos1, pos2 - pos1 ) );
      pos1 = pos2 + 1;
      pos2 = in_text.find( '.', pos1 );
   }

   out_data.push_back( in_text.substr( pos1 ) );
}
// end of split

//*******************************************************************
// is_decnumber                                              PRIVATE
//*******************************************************************
bool BTCallArgs::is_decnumber( const string& in_text ) const
{
   bool retval = true;

   string::const_iterator it = in_text.begin();
   const string::const_iterator end = in_text.end();
   while( it != end ) {
      if( !isdigit( *it ) ) {
         retval = false;
         break;
      }
      ++it;
   }

   return retval;
}
// end of is_decnumber
