/********************************************************************
 * Copyright (C) 2007 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Beesoft Tracer.
 *
 * Beesoft Tracer is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Beesoft Tracer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Beesoft Tracer
 * FILE   : BTCallArgs.h
 * DATE   : 27.06.2007
 *******************************************************************/
#ifndef INCLUDED_BTCallArgs_h
#define INCLUDED_BTCallArgs_h

/*------- include files:
------------------------------------------------------------------*/
#include <string>
#include <vector>

/*------- class declaration:
-------------------------------------------------------------------*/
class BTCallArgs
{
// ******* CONSTRUCTION / DESTRUCTION *******
public:
   BTCallArgs( int, const char** );
   virtual ~BTCallArgs() {};
private:
   BTCallArgs( const BTCallArgs& );
   BTCallArgs& operator=( const BTCallArgs& );

// ******* CONTANTS *******
private:
   static const int         DEFAULT_PORT;
   static const std::string DEFAULT_ADDR;
   static const std::string ADDR_MARKER;
   static const std::string PORT_MARKER;
   static const std::string HELP1;
   static const std::string HELP2;
   
// ******* MEMBERS *******
private:
   int port_;
   std::string addr_;
   bool help_;

// ******* METHODS *******
public:
   int               port () const { return port_; }
   const std::string& addr() const { return addr_; }
   bool     port_is_valid () const { return ( port_ > 0 );  }
   bool     addr_is_valid () const { return !addr_.empty(); }
   bool     need_help     () const { return help_; }
private:
   void split       ( const std::string&, std::vector<std::string>& ) const;
   bool is_decnumber( const std::string& ) const;
   bool is_ip4addr  ( const std::string& ) const;
};

#endif // INCLUDED_BTCallArgs_h
