/********************************************************************
 * Copyright (C) 2007 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Beesoft Tracer.
 *
 * Beesoft Tracer is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Beesoft Tracer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Beesoft Tracer; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Beesoft Tracer
 * FILE   : BTCondWait.cpp
 * DATE   : 04.07.2007
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include <BTCondWait.h>
#include <BTMutex.h>
#include <errno.h>
#include <iostream>
using namespace std;

//*******************************************************************
// BTCondWait                                            CONSTRUCTOR
//*******************************************************************
BTCondWait::BTCondWait()
{
   const int status = pthread_cond_init( &cond_, 0 );
   if( status ) {
      cerr << "BTCondWait::BTCondWait - " << strerror( status ) << endl;
   }
}
// end of BTCondWait

//*******************************************************************
// ~BTCondWait                                            DESTRUCTOR
//*******************************************************************
BTCondWait::~BTCondWait()
{
   const int status = pthread_cond_destroy( &cond_ );
   if( status ) {
      cerr << "BTCondWait::~BTCondWait - " << strerror( status ) << endl;
   }
}
// end of ~BTCondWait

//*******************************************************************
// wait                                                       PUBLIC
//*******************************************************************
void BTCondWait::wait( BTMutex* const in_mutex )
{
   const int status = pthread_cond_wait( &cond_, &in_mutex->mutex_ );
   if( status ) {
      cerr << "BTCondWait::wait - " << strerror( status ) << endl;
   }
}
// end of wait

//*******************************************************************
// wake_up                                                    PUBLIC
//*******************************************************************
void BTCondWait::wake_up()
{
   const int status = pthread_cond_signal( &cond_ );
   if( status ) {
      cerr << "BTCondWait::wake_up - " << strerror( status ) << endl;
   }
}
// end of wake_up

//*******************************************************************
// wake_up_all                                                PUBLIC
//*******************************************************************
void BTCondWait::wake_up_all()
{
   const int status = pthread_cond_broadcast( &cond_ );
   if( status ) {
      cerr << "BTCondWait::wake_up_all - " << strerror( status ) << endl;
   }
}
// end of wake_up

//*******************************************************************
// reset                                                      PUBLIC
//*******************************************************************
void BTCondWait::reset()
{
   {
      const int status = pthread_cond_destroy( &cond_ );
      if( status ) {
         cerr << "BTCondWait::reset:destroy - " << strerror( status ) << endl;
      }
   }
   {
      const int status = pthread_cond_init( &cond_, 0 );
      if( status ) {
         cerr << "BTCondWait::reset:init - " << strerror( status ) << endl;
      }
   }
}
// end of reset
