/********************************************************************
 * Copyright (C) 2007 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Beesoft Tracer.
 *
 * Beesoft Tracer is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Beesoft Tracer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Beesoft Tracer; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Beesoft Tracer
 * FILE   : BTMutex.cpp
 * DATE   : 27.06.2007
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include <BTMutex.h>
#include <errno.h>
#include <iostream>
using namespace std;


//*******************************************************************
// BTMutex                                               CONSTRUCTOR
//*******************************************************************
BTMutex::BTMutex()
: name_ ( "noname" )
{
   init();
}
BTMutex::BTMutex( const string& in_name )
: name_ ( in_name )
{
   init();
}
// end of BTMutex

//*******************************************************************
// init                                                      PRIVATE
//*******************************************************************
void BTMutex::init()
{
   const int status = pthread_mutex_init( &mutex_, 0 );
   if( status ) {
       cerr << "BTMutex::BTMutex (" << name_ << ") - " << strerror( status ) << endl;
   }
}
// end of init

//*******************************************************************
// ~BTMutex                                               DESTRUCTOR
//*******************************************************************
BTMutex::~BTMutex()
{
   const int status = pthread_mutex_destroy( &mutex_ );
   if( status ) {
       cerr << "BTMutex::~BTMutex (" << name_ <<  ") - " << strerror( status ) << endl;
   }
}
// end of BTMutex

//*******************************************************************
// set_name                                                   PUBLIC
//*******************************************************************
void BTMutex::set_name( const string& in_name )
{
   name_ = in_name;
}
// end of set_name

//*******************************************************************
// lock                                                       PUBLIC
//*******************************************************************
bool BTMutex::lock()
{
   bool retval = true;

   const int status = pthread_mutex_lock( &mutex_ );
   if( status ) {
      retval = false;
      cerr << "BTMutex::lock (" << name_ << ") - " << strerror( status ) << endl;
   }
   return retval;
}
// end of lock

//*******************************************************************
// try_lock                                                   PUBLIC
//*******************************************************************
bool BTMutex::try_lock()
{
   bool retval = true;

   const int status = pthread_mutex_trylock( &mutex_ );
   if( status ) {
      retval = false;
      if( status != EBUSY ) {
         cerr << "BTMutex::try_lock (" << name_ << ") - " << strerror( status ) << endl;
      }
   }
   return retval;
}
// end of try_lock

//*******************************************************************
// unlock                                                     PUBLIC
//*******************************************************************
bool BTMutex::unlock()
{
   bool retval = true;

   const int status = pthread_mutex_unlock( &mutex_ );
   if( status ) {
      cerr << "BTMutex::unlock (" << name_ << ") - " << strerror( status ) << endl;
   }
}
// end of unlock
