/********************************************************************
 * Copyright (C) 2007 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Beesoft Tracer.
 *
 * Beesoft Tracer is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Beesoft Tracer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Beesoft Tracer
 * FILE   : BTMutex.cpp
 * DATE   : 27.06.2007
 *******************************************************************/
#ifndef INCLUDED_BTMutex_h
#define INCLUDED_BTMutex_h

/*------- include files:
-------------------------------------------------------------------*/
#include <pthread.h>
#include <string>

/*------- class declaration:
-------------------------------------------------------------------*/
class BTMutex
{
// ******* CONTRUCTION / DESTRUCTION *******
public:
   BTMutex();
   BTMutex( const std::string& );
   virtual ~BTMutex();
private:
   BTMutex( const BTMutex& );
   BTMutex& operator=( const BTMutex& );

// ******* MEMBERS *******
private:
   std::string     name_;
   pthread_mutex_t mutex_;

// ******* METHODS *******
private:
   void init     ();
public:
   bool lock     ();
   bool try_lock ();
   bool unlock   ();
   void set_name ( const std::string& );

   friend class BTCondWait;
};

class BTScopeMutex : public BTMutex
{
// ******* CONSTRUCTION / DESTRUCTION *******
public:
   BTScopeMutex() : BTMutex() {
      lock();
   }
   ~BTScopeMutex() {
      unlock();
   }
private:
   BTScopeMutex( const BTScopeMutex& );
   BTScopeMutex& operator=( const BTScopeMutex& );
};

#endif // INCLUDED_BTMutex_h
