/********************************************************************
 * Copyright (C) 2007 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Beesoft Tracer.
 *
 * Beesoft Tracer is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Beesoft Tracer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Beesoft Tracer
 * FILE   : BTSocket.cpp
 * DATE   : 27.06.2007
 *******************************************************************/
#ifndef INCLUDED_BTSocket_h
#define INCLUDED_BTSocket_h

/*------- include files:
-------------------------------------------------------------------*/
#include <string>


/*------- class declarations:
-------------------------------------------------------------------*/
class BTSocketInterface
{
public:
   virtual void bind    () = 0;
   virtual void connect () = 0;
   virtual int  accept  () = 0;
};

class BTSocketException
{
// ******* CONSTRUCTION / DESTRUCTION *******
public:
   BTSocketException( const std::string& in_text ) : text_( in_text )
   {}
   virtual ~BTSocketException()
   {}
   BTSocketException( const BTSocketException& rhs ) {
      text_ = rhs.text_;
   }
private:
   BTSocketException& operator=( const BTSocketException& );
   
// ******* MEMBERS *******
private:
   std::string text_;

// ******* METHODS *******
public:
   const std::string& what() { return text_; }
};

class BTSocket
{
// ******* TYPES *******
public:
   typedef enum {
      UNKNOWN_TYPE = 0,
      SERVER,
      CLIENT,
      CHILD
   } Type;

   typedef enum Domain {
      UNKNOWN_DOMAIN = -1,
      UNIX_DOMAIN,
      IPv4,
      IPv6
   };

// ******* CONSTRUCTION / DESTRUCTION *******
public:
   BTSocket();
   BTSocket( int );
   BTSocket( Domain );
   virtual ~BTSocket() {}

// ******* CONSTANTS *******
private:
   enum {
      BUFFER_SIZE = 1024,
   };
protected:
   enum {
      INVALID_SOCKET = -1
   };
   
// ******* MEMBERS *******
private:
   int  fd_;

// ******* METHODS *******
public:
   void close    ();
   bool is_valid () const;
   int  fd       () const;
   bool set_fd   ( int );
   void listen   ();
   bool read     ( std::string&, int );
   int  read     ( void*, int );
   bool write    ( const std::string& );
   int  write    ( const void*, int );

   template <typename T>
   bool read( T& out_data ) {
      return ( sizeof( T ) == read( &out_data, sizeof( T ) ) );
   }
   template<typename T>
   bool write( const T& in_data ) {
      return ( sizeof( T ) == write( &in_data, sizeof( T ) ) );
   }
};

//*******************************************************************
// is_valid                                            PUBLIC inline
//*******************************************************************
inline bool BTSocket::is_valid() const
{
   return ( fd_ != INVALID_SOCKET );
}
// end of is_valid

//*******************************************************************
// fd                                                  PUBLIC inline
//*******************************************************************
inline int BTSocket::fd() const {
   return fd_;
}
// end of fd

//*******************************************************************
// set_fd                                              PUBLIC inline
//*******************************************************************
inline bool BTSocket::set_fd( const int in_fd ) {
   bool retval = false;

   if( INVALID_SOCKET == fd_ ) {
      fd_ = in_fd;
      retval = true;
   }
   return retval;
}
// end of set_fd

#endif // INCLUDED_BTSocket_h
