/********************************************************************
 * Copyright (C) 2007 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Beesoft Tracer.
 *
 * Beesoft Tracer is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Beesoft Tracer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Beesoft Tracer
 * FILE   : BTTcpSocket.cpp
 * DATE   : 27.06.2007
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include <BTTcpSocket.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>
#include <iostream>
using namespace std;

//*******************************************************************
// BTTcpSocket                                           CONSTRUCTOR
//*******************************************************************
BTTcpSocket::BTTcpSocket( const int in_fd )
: BTSocket( in_fd )
, BTSocketInterface()
, addr_ ( "" )
, port_ ( -1 )
, type_ ( CHILD )
{}
BTTcpSocket::BTTcpSocket( const string& in_addr, const int in_port, const Type in_type )
: BTSocket( BTSocket::IPv4 )
, addr_ ( in_addr )
, port_ ( in_port )
, type_ ( in_type )
{
   if( SERVER == in_type ) {
      int opc = 1;
      setsockopt( fd(), SOL_SOCKET, SO_KEEPALIVE, &opc, sizeof( int ) );
   }
}
// end of BTTcpSocket

//*******************************************************************
// bind                                             PUBLIC inherited
//*******************************************************************
void BTTcpSocket::bind()
{
   struct sockaddr_in saddr;
   memset( &saddr, 0,sizeof( saddr ) );
   saddr.sin_family      = AF_INET;
   saddr.sin_addr.s_addr = htonl( INADDR_ANY );
   saddr.sin_port        = htons( port_ );
      
   if( ::bind( fd(), reinterpret_cast< struct sockaddr* >( &saddr ), sizeof( saddr ) ) ) {
      throw BTSocketException( string( strerror( errno ) ) );
   }
}
// end of bind

//*******************************************************************
// connect                                          PUBLIC inherited
//*******************************************************************
void BTTcpSocket::connect()
{
   struct sockaddr_in caddr;
   memset( &caddr, 0, sizeof( caddr ) );
   caddr.sin_family = AF_INET;
   caddr.sin_port   = htons( port_ );
   inet_pton( AF_INET, addr_.c_str(), &caddr.sin_addr );

   if( ::connect( fd(), reinterpret_cast< struct sockaddr* >( &caddr ), sizeof( caddr ) ) ) {
      throw BTSocketException( string( strerror( errno ) ) );
   }
}
// end of connect

//*******************************************************************
// accept                                           PUBLIC inherited
//*******************************************************************
int BTTcpSocket::accept()
{
   int retval = INVALID_SOCKET;
   
   struct sockaddr_in saddr;
   memset( &saddr, 0, sizeof( saddr ) );
   socklen_t socklen = sizeof( saddr );
   retval = ::accept( fd(), reinterpret_cast< sockaddr* >( &saddr ), &socklen );
   if( INVALID_SOCKET == retval ) {
      throw BTSocketException( string( strerror( errno ) ) );
   }
   return retval;
}
// end of connect
