/********************************************************************
 * Copyright (C) 2007 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Beesoft Tracer.
 *
 * Beesoft Tracer is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Beesoft Tracer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Beesoft Tracer
 * FILE   : BTThread.cpp
 * DATE   : 27.06.2007
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include <BTThread.h>
#include <iostream>
using namespace std;


//*******************************************************************
// BTThread                                              CONSTRUCTOR
//*******************************************************************
BTThread::BTThread()
: started_( false )
{
}
// end of BTThread

//*******************************************************************
// ~BTThread                                              DESTRUCTOR
//*******************************************************************
BTThread::~BTThread()
{}
// end of ~BTThread

//*******************************************************************
// start                                                      PUBLIC
//*******************************************************************
bool BTThread::start()
{
   bool retval = true;
   const int status = pthread_create( &id_, 0, do_it, this );

   if( status ) {
      retval = false;
      cerr << "BTThread::start - " << strerror( status ) << endl;
   }
   return retval;
}
// end of start

//*******************************************************************
// join                                                       PUBLIC
//*******************************************************************
void BTThread::join()
{
   const int status = pthread_join( id_, 0 );
   if( status ) {
      cerr << "BTThread::join - " << strerror( status ) << endl;
   }
}
// end of join

//*******************************************************************
// do_it                                                     PRIVATE
//*******************************************************************
void* BTThread::do_it( void* in_ptr )
{
   // prolog
   BTThread* const my_this = static_cast< BTThread* >( in_ptr );
   if( 0 == my_this ) pthread_exit( 0 );
   my_this->started_ = true;

   my_this->worker();
   
   // epilog
   my_this->started_ = false;
   pthread_exit( 0 );
}
// end of do_it

//*******************************************************************
// cancel                                                     PUBLIC
//*******************************************************************
void BTThread::cancel()
{
   const int status = pthread_cancel( id_ );
   if( status ) {
      cerr << "BTThread::cancel - " << strerror( status ) << endl;
   }
}
// end of cancel
