/********************************************************************
 * Copyright (C) 2007 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Beesoft Tracer.
 *
 * Beesoft Tracer is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Beesoft Tracer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Beesoft Tracer
 * FILE   : BTTraceMessage.cpp
 * DATE   : 27.06.2007
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include <BTTraceMessage.h>
#include <BTUnixSocket.h>
#include <iomanip>
using namespace std;

/*------- local constants:
-------------------------------------------------------------------*/
const char BTTraceMessage::SEPARATOR = '|';


//*******************************************************************
// split                                                      PUBLIC
//*******************************************************************
void BTTraceMessage::split( vector<string>& out_data ) const
{
   out_data.clear();
   
   unsigned int pos1 = 0;
   unsigned int pos2 = text_.find( SEPARATOR );

   while( pos2 != string::npos ) {
      out_data.push_back( text_.substr( pos1, pos2 - pos1 ) );
      pos1 = pos2 + 1;
      pos2 = text_.find( SEPARATOR, pos1 );
   }

   out_data.push_back( text_.substr( pos1 ) );
}
// end of split

//*******************************************************************
// read                                                       PUBLIC
//*******************************************************************
bool BTTraceMessage::read( BTUnixSocket* const in_socket )
{
   bool retval = false;
   int  n;

   clear();
   if( in_socket->read<int>( type_ ) ) {
      if( Tracer::is_valid_type( type_ ) ) {
         if( in_socket->read<int>( n ) ) {
            if( in_socket->read( text_, n ) ) {
               retval = true;
            }
         }
      }
   }
   return retval;
}
// end of read

//*******************************************************************
// write                                                      PUBLIC
//*******************************************************************
bool BTTraceMessage::write( BTUnixSocket* const in_socket ) const
{
   bool retval = false;

   if( in_socket->write<int>( type_ ) ) {
      const int n = text_.length();
      if( in_socket->write<int>( n ) ) {
         if( in_socket->write( text_.c_str(), n ) ) {
            retval = true;
         }
      }
   }
   return retval;
}
// end of write

//*******************************************************************
// print                                                      PUBLIC
//*******************************************************************
void BTTraceMessage::print() const
{
   cout << setw( 10 ) << left << Tracer::HEADER[ type_ ] << ": " << scope_id_ << ", " << text_ << endl;
}
// end of print
