/********************************************************************
 * Copyright (C) 2007 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Beesoft Tracer.
 *
 * Beesoft Tracer is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Beesoft Tracer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Beesoft Tracer
 * FILE   : BTTraceMessage.cpp
 * DATE   : 27.06.2007
 *******************************************************************/
#ifndef INCLUDED_BTTraceMessage_h
#define INCLUDED_BTTraceMessage_h

/*------- include files:
-------------------------------------------------------------------*/
#include <string>
#include <vector>
#include <Tracer.h>

/*------- forward declarations:
-------------------------------------------------------------------*/
class BTUnixSocket;

/*------- class declaration:
-------------------------------------------------------------------*/
class BTTraceMessage
{
// ******* CONSTRUCTION / DESTRUCTION ******
public:
   BTTraceMessage()
   : scope_id_( -1 ), type_( Tracer::INVALID_TYPE ), text_( "" )
   {}
   BTTraceMessage( const int in_type )
   : scope_id_( -1 ), type_( in_type ), text_( "" )
   {}
   BTTraceMessage( const int in_type, const std::string& in_text )
   : scope_id_( -1 ), type_( in_type ), text_( in_text )
   {}
   BTTraceMessage( const int in_scope_id, const int in_type, const std::string& in_text )
   : scope_id_( in_scope_id ), type_( in_type ), text_( in_text )
   {}
   virtual ~BTTraceMessage()
   {}
   BTTraceMessage( const BTTraceMessage& rhs ) {
      scope_id_ = rhs.scope_id_;
      type_     = rhs.type_;
      text_     = rhs.text_;
   }
   BTTraceMessage& operator=( const BTTraceMessage& rhs ) {
      if( this != &rhs ) {
         scope_id_ = rhs.scope_id_;
         type_     = rhs.type_;
         text_     = rhs.text_;
      }
      return *this;
   }

// ******* CONSTANTS *******
public:
   static const char SEPARATOR;
   
// ******* MEMBERS *******
private:
   int         scope_id_;
   int         type_;
   std::string text_;

// ******* METHODS *******
public:
   int scope_id() const                    { return scope_id_; }
   void scope_id( const int in_scope_id )  { scope_id_ = in_scope_id; }
   int type() const                        { return type_;    }
   const std::string& text() const         { return text_;    }
   void type( const int in_type )          { type_ = in_type; }
   void text( const std::string& in_text ) { text_ = in_text; }
   void clear()                            { type_ = Tracer::INVALID_TYPE; text_.clear(); }

   bool read  ( BTUnixSocket* );
   bool write ( BTUnixSocket* ) const;
   void split ( std::vector<std::string>& ) const;
   void print () const;
};

#endif // INCLUDED_BTTraceMessage_h
