/********************************************************************
 * Copyright (C) 2007 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Beesoft Tracer.
 *
 * Beesoft Tracer is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Beesoft Tracer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Beesoft Tracer
 * FILE   : BTUnixSocket.cpp
 * DATE   : 27.06.2007
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include <BTUnixSocket.h>
#include <string>
#include <errno.h>
#include <sys/un.h>
#include <sys/socket.h>
using namespace std;


//*******************************************************************
// BTUnixSocket                                          CONSTRUCTOR
//*******************************************************************
BTUnixSocket::BTUnixSocket( const int in_fd )
: BTSocket( in_fd )
, BTSocketInterface()
, type_ ( CHILD )
, path_ ( "" )
{}
BTUnixSocket::BTUnixSocket( const std::string& in_path, const Type in_type )
: BTSocket          ( BTSocket::UNIX_DOMAIN )
, BTSocketInterface ()
, type_             ( in_type )
, path_             ( in_path )
{
   if( SERVER == type_ ) {
      ::unlink( path_.c_str() );
   }
}
// end of BTUnixSocket

//*******************************************************************
// bind                                                       PUBLIC
//*******************************************************************
void BTUnixSocket::bind()
{
   struct sockaddr_un addr;
   memset( &addr, 0, sizeof( addr ) );
   addr.sun_family = AF_LOCAL;
   strncpy( addr.sun_path, path_.c_str(), sizeof( addr.sun_path ) - 1 );

   if( ::bind( fd(), reinterpret_cast< struct sockaddr* >( &addr ), SUN_LEN( &addr ) ) ) {
      throw BTSocketException( string( strerror( errno ) ) );
   }
}
// end of bind

//*******************************************************************
// connect                                                    PUBLIC
//*******************************************************************
void BTUnixSocket::connect()
{
   struct sockaddr_un addr;
   memset( &addr, 0, sizeof( addr ) );
   addr.sun_family = AF_LOCAL;
   strncpy( addr.sun_path, path_.c_str(), sizeof( addr.sun_path ) - 1 );

   if( ::connect( fd(), reinterpret_cast< struct sockaddr* >( &addr ), SUN_LEN( &addr ) ) ) {
      throw BTSocketException( string( strerror( errno ) ) );
   }
}
// end of connect

//*******************************************************************
// accept                                                     PUBLIC
//*******************************************************************
int BTUnixSocket::accept()
{
   int retval = INVALID_SOCKET;

   struct sockaddr_un addr;
   memset( &addr, 0, sizeof( addr ) );
   socklen_t socklen = sizeof( addr ); //SUN_LEN( &addr );
   retval = ::accept( fd(), reinterpret_cast< sockaddr* >( &addr ), &socklen );
   if( INVALID_SOCKET == retval ) {
      throw BTSocketException( string( strerror( errno ) ) );
   }
   return retval;
}
// end of accept
