/********************************************************************
 * Copyright (C) 2007 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Beesoft Tracer.
 *
 * Beesoft Tracer is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Beesoft Tracer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Beesoft Tracer; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Beesoft Tracer
 * FILE   : Tracer.cpp
 * DATE   : 27.06.2007
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include <Tracer.h>
#include <BTTraceMessage.h>
#include <BTUnixSocket.h>
#include <iostream>
using namespace std;

/*------- static variables:
-------------------------------------------------------------------*/
Tracer* Tracer::tracer_ = 0;

/*------- local constants:
-------------------------------------------------------------------*/
const char* const Tracer::UNIX_DEV  = "/tmp/tracer";
const string      Tracer::HEADER[]  = { "INVALID", "MSG", "WARNING", "ERROR", "SCOPE_IN", "SCOPE_OUT", "SCOPE_DEF" };


//*******************************************************************
// Tracer                                        CONSTRUCTOR private
//*******************************************************************
Tracer::Tracer()
: socket_( new BTUnixSocket( UNIX_DEV ) )
{
   socket_->connect();
}
// end of Tracer

//*******************************************************************
// ~Tracer                                        DESTRUCTOR private
//*******************************************************************
Tracer::~Tracer()
{
}
// end of ~Tracer

//*******************************************************************
// instance                                            PUBLIC static
//*******************************************************************
Tracer* const Tracer::instance()
{
   if( 0 == tracer_ ) {
      tracer_ = new Tracer;
   }
   return tracer_;
}
// end of instance

//*******************************************************************
// is_valid_type                                       PUBLIC static
//*******************************************************************
bool Tracer::is_valid_type ( const int in_type )
{
   return ( ( in_type >= MSG ) && ( in_type < END_MARKER ) );
}
// end of is_valid_type

//*******************************************************************
// send                                                       PUBLIC
//*******************************************************************
// 1. wersja dla MSG, WARNING, ERROR
void Tracer::send(   const int     in_type,
                     const string& in_class,
                     const string& in_method,
                     const string& in_text )
{
   mutex_.lock();
   
   if( socket_ ) {
      const int idx = in_text.find( '\n' );

      string msg_body = in_class;
      msg_body += BTTraceMessage::SEPARATOR;
      msg_body += in_method;
      msg_body += BTTraceMessage::SEPARATOR;
      msg_body += ( string::npos == idx ) ? in_text : in_text.substr( 0, idx );

      BTTraceMessage tm( in_type, msg_body );
      tm.write( socket_ );
   }

   mutex_.unlock();
}
// 2. wersja dla SCODE_DEF, SCOPE_IN, SCOPE_OUT
void Tracer::send( const int in_type, const string& in_text1, const string& in_text2 )
{
   mutex_.lock();
   
   if( socket_ ) {
      string msg_body = in_text1;
      msg_body += BTTraceMessage::SEPARATOR;
      msg_body += in_text2;
      
      BTTraceMessage tm( in_type, msg_body );
      tm.write( socket_ );
   }

   mutex_.unlock();
}
// end of send
