/********************************************************************
 * Copyright (C) 2007 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Beesoft Tracer.
 *
 * Beesoft Tracer is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Beesoft Tracer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Beesoft Tracer
 * FILE   : DataBase.cpp
 * DATE   : 27.06.2007
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "DataBase.h"
#include <vector>
#include <Tracer.h>
#include <BTMutex.h>
using namespace std;

/*------- local variables:
-------------------------------------------------------------------*/
DataBase*     DataBase::instance_    = 0;


//*******************************************************************
// instance                                            PUBLIC static
//*******************************************************************
DataBase* const DataBase::instance()
{
   if( 0 == instance_ ) {
      instance_ = new DataBase();
   }
   return instance_;
}
// end of instance

//*******************************************************************
// add                                                        PUBLIC
//*******************************************************************
void DataBase::add( const BTTraceMessage& in_data )
{
   mutex_.lock();

   vector<string> items;
   in_data.split( items );

   if( Tracer::SCOPE_DEF == in_data.type() ) {
      scope_defs_.add( ScopeInfo( items[ 0 ], items[ 1 ] ) );
   }
   else {
      const int id = scope_defs_.scope_id( items[ 0 ], items[ 1 ] );
      msgs_.push_back( BTTraceMessage( id, in_data.type(), ( items.size() > 2 ) ? items[ 2 ] : "" ) );
   }
   not_empty_.wake_up();
   //print_info();
   mutex_.unlock();
}
// end of add

//*******************************************************************
// get                                                        PUBLIC
//*******************************************************************
void DataBase::get( BTTraceMessage& out_data )
{
   mutex_.lock();
   while( msgs_.empty() ) {
      not_empty_.wait( &mutex_ );
   }
   out_data = msgs_.front();
   msgs_.pop_front();
   mutex_.unlock();
}
// end of get

//*******************************************************************
// reset                                                      PUBLIC
//*******************************************************************
void DataBase::reset()
{
   scope_defs_.clear();
   msgs_.clear();
}
// end of reset

//*******************************************************************
// print_info                                                 PUBLIC
//*******************************************************************
void DataBase::print_info() const
{
   cout << "-------------------------------------------------------" << endl;
   cout << "SCOPE DEFS:" << endl;
   scope_defs_.print_info();
   cout << endl;

   cout << "MESSAGES, WARNING AND ERRORS:" << endl;
   MsgQueue::const_iterator it = msgs_.begin();
   const MsgQueue::const_iterator end = msgs_.end();
   while( it != end ) {
      it->print();
      ++it;
   }
   cout << endl;
}
// end of print_info
