/********************************************************************
 * Copyright (C) 2007 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Beesoft Tracer.
 *
 * Beesoft Tracer is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Beesoft Tracer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Beesoft Tracer
 * FILE   : DataBase.cpp
 * DATE   : 27.06.2007
 *******************************************************************/
#ifndef INCLUDED_DataBase_h
#define INCLUDED_DataBase_h

/*------- include files:
-------------------------------------------------------------------*/
#include <BTTraceMessage.h>
#include <BTMutex.h>
#include <BTCondWait.h>
#include <ScopeDefs.h>
#include <deque>

/*------- class declaration:
-------------------------------------------------------------------*/
class DataBase
{
// ******* TYPES *******
private:
   typedef std::deque<BTTraceMessage> MsgQueue;

// ******* CONSTRUCTION / DESTRUCTION *******
private:
   DataBase() {}
   virtual ~DataBase() {}
   DataBase( const DataBase& );
   DataBase& operator=( const DataBase& );

// ******* MEMBERS *******
private:
   static DataBase* instance_;
   ScopeDefs        scope_defs_;
   MsgQueue         msgs_;
   BTMutex          mutex_;
   BTCondWait       not_empty_;

// ******* METHODS *******
public:
   static DataBase* const     instance  ();
   void                       add       ( const BTTraceMessage& );
   void                       get       ( BTTraceMessage& );
   const ScopeDefs::ItemsMap& scopes    () const { return scope_defs_.scopes(); }
   void                       print_info() const;
   void                       reset     ();
};

#endif //  INCLUDED_DataBase_h
