/********************************************************************
 * Copyright (C) 2007 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Beesoft Tracer.
 *
 * Beesoft Tracer is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Beesoft Tracer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Beesoft Tracer
 * FILE   : TcpListener.cpp
 * DATE   : 27.06.2007
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "TcpListener.h"
#include <BTTraceMessage.h>
#include "DataBase.h"
#include <iostream>
using namespace std;

/*------- local constants:
-------------------------------------------------------------------*/
const int TcpListener::DEFAULT_PORT = 6789;


//*******************************************************************
// TcpListener                                           CONSTRUCTOR
//*******************************************************************
TcpListener::TcpListener( const int in_port )
: BTThread()
, socket_ ( BTTcpSocket( "", in_port, BTTcpSocket::SERVER ) )
{}
// end of TcpListener

//*******************************************************************
// worker                                          PRIVATE inherited
//*******************************************************************
void TcpListener::worker()
{
   try {
      socket_.bind();
      socket_.listen();
      while( true ) {
         send_data( socket_.accept() );
      }
   }
   catch( BTSocketException& in_exception ) {
      cerr << "TcpListener: " << in_exception.what() << endl;
   }
}
// end of worker

//*******************************************************************
// send_data                                                 PRIVATE
//*******************************************************************
void TcpListener::send_data( const int in_fd )
{
   BTTcpSocket tcps( in_fd );
   
   try {
      if( send_scopes( tcps ) ) {
         send_msg   ( tcps );
      }
      tcps.close();
   }
   catch( ... ) {
      tcps.close();
   }
}
// end of send_data

//*******************************************************************
// send_scopes                                               PRIVATE
//*******************************************************************
bool TcpListener::send_scopes( BTTcpSocket& in_socket )
{
   bool retval = false;
   
   if( !DataBase::instance()->scopes().empty() ){
      retval = true;
      
      bool loop = true;
      const ScopeDefs::ItemsMap scopes = DataBase::instance()->scopes();
      ScopeDefs::ItemsMap::const_iterator it = scopes.begin();
      const ScopeDefs::ItemsMap::const_iterator end = scopes.end();
      while( loop && ( it != end ) ) {
         loop = false;
         if( in_socket.write<int>( static_cast<int>( Tracer::SCOPE_DEF ) ) ) {
            if( in_socket.write<int>( static_cast<int>( it->second ) ) ) {
               string msg_body = it->first.class_;
               msg_body += BTTraceMessage::SEPARATOR;
               msg_body += it->first.method_;
               if( in_socket.write<int>( msg_body.length() ) ) {
                  if( in_socket.write( msg_body ) ) {
                     loop = true;
                  }
               }
            }
         }
         ++it;
      }
   }
   return retval;
}
// end of send_scopes

//*******************************************************************
// send_msg                                                  PRIVATE
//*******************************************************************
void TcpListener::send_msg( BTTcpSocket& in_socket )
{
   bool loop = true;
   BTTraceMessage tm;

   while( loop ) {
      DataBase::instance()->get( tm );
      loop = false;
      if( in_socket.write<int>( tm.type() ) ) {
         if( in_socket.write<int>( tm.scope_id() ) ) {
            if( send_next( tm.type() ) ) {
               if( in_socket.write<int>( static_cast<int>( tm.text().length() ) ) ) {
                  if( in_socket.write( tm.text() ) ) {
                     loop = true;
                  }
               }
            }
            else {
               loop = true;
            }
         }
      }
   }
}
// end of send_msg

//*******************************************************************
// send_next                                                 PRIVATE
//*******************************************************************
bool TcpListener::send_next( const int in_type ) const
{
   return !( ( Tracer::SCOPE_IN == in_type ) || ( Tracer::SCOPE_OUT == in_type ) );
}
// end of send_next
