/********************************************************************
 * Copyright (C) 2007 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Beesoft Tracer.
 *
 * Beesoft Tracer is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Beesoft Tracer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Beesoft Tracer
 * FILE   : UxListener.cpp
 * DATE   : 27.06.2007
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "UxListener.h"
#include "DataBase.h"
#include <BTTraceMessage.h>
#include <iostream>
#include <iomanip>
using namespace std;

/*------- local constants:
-------------------------------------------------------------------*/
const char* const UxListener::UNIX_FILE = "/tmp/tracer";


//*******************************************************************
// UxListener                                            CONSTRUCTOR
//*******************************************************************
UxListener::UxListener()
: BTThread    ()
, socket_     ( BTUnixSocket( UNIX_FILE, BTUnixSocket::SERVER ) )
, after_scopes( false )
{}
// end of Listener

//*******************************************************************
// worker                                          PRIVATE inherited
//*******************************************************************
void UxListener::worker()
{
   try {
      socket_.bind();
      socket_.listen();
      read_data( socket_.accept() );
   }
   catch( BTSocketException& in_exception ) {
      cerr << "UxListener: " << in_exception.what() << endl;
   }
}
// end of worker

//*******************************************************************
// read_data                                                 PRIVATE
//*******************************************************************
void UxListener::read_data( const int in_fd )
{
   BTUnixSocket uxs( in_fd );

   try {
      BTTraceMessage tm;
      while( tm.read( &uxs ) ) {
         if( Tracer::SCOPE_DEF == tm.type() ) {
            if( after_scopes ) {
               DataBase::instance()->reset();
               after_scopes = false;
            }
         }
         else {
            after_scopes = true;
         }
         
         DataBase::instance()->add( tm );
      }
      uxs.close();
   }
   catch( ... ) {
      uxs.close();
      throw;
   }
}
// end of read_data
