/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : Event.h
 * DATE   : 3.09.2006
 *******************************************************************/
#ifndef INCLUDED_Event_h
#define INCLUDED_Event_h

/*------- include files:
-------------------------------------------------------------------*/
#include <QEvent>
#include <QString>
#include <QVariant>
#include <QMutex>
#include <QVector>

/*------- class declaration:
-------------------------------------------------------------------*/
class Event : public QEvent
{
// ******* TYPES *******
public:
   enum {
      SCOPES_READY = ( QEvent::User + 1 ),
      GET_SELECTED_SCOPES,
      SCOPES_SELECTED,
      DISPLAY_MESSAGE,
      CONNECTED,
      DISCONNECTED,
      SOCKET_STATE,
   };

public:
   explicit Event( int const in_event_id )
   : QEvent( static_cast<QEvent::Type>( in_event_id ))
   {
      d_data.reserve( 0 );
   }
   explicit Event( const int in_event_id, const QVariant in_val )
   : QEvent( static_cast<QEvent::Type>( in_event_id ) )
   {
      d_data.reserve( 1 );
      d_data << in_val;
   }
   explicit Event( const int in_event_id, const QVariant in_val1, const QVariant in_val2 )
   : QEvent( static_cast<QEvent::Type>( in_event_id ) )
   {
      d_data.reserve( 2 );
      d_data << in_val1 << in_val2;
   }
   explicit Event( const int in_event_id, const QVariant in_val1, const QVariant in_val2, const QVariant in_val3 )
   : QEvent( static_cast<QEvent::Type>( in_event_id ) )
   {
      d_data.reserve( 3 );
      d_data << in_val1 << in_val2 << in_val3;
   }
   explicit Event( const int in_event_id, const QVariant in_val1, const QVariant in_val2, const QVariant in_val3, const QVariant in_val4 )
   : QEvent( static_cast<QEvent::Type>( in_event_id ) )
   {
      d_data.reserve( 4 );
      d_data << in_val1 << in_val2 << in_val3 << in_val4;
   }
public:
   QVector< QVariant > d_data;
private:
   Event( const Event& );
   Event& operator=( const Event& );
};

#endif // INCLUDED_Event_h
