/********************************************************************
 * Copyright (C) 2006-2007 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Beesoft Tracer.
 *
 * Beesoft Tracer is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Beesoft Tracer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Beesoft Tracer; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Beesoft Tracer
 * FILE   : MainWindow.h
 * DATE   : 29.06.2007
 *******************************************************************/
#ifndef INCLUDED_MainWindow_h
#define INCLUDED_MainWindow_h

/*------- include files:
-------------------------------------------------------------------*/
#include <QMainWindow>

/*------- include files:
-------------------------------------------------------------------*/
class Table;
class Tree;
class QDockWidget;
class QMenu;
class QAction;
class QToolbar;
class Network;
class Monitor;

/*------- class declaration:
-------------------------------------------------------------------*/
class MainWindow : public QMainWindow
{
   Q_OBJECT

// ******* CONSTRUCTION / DESTRUCTION *******
public:
   MainWindow();
   ~MainWindow();
private:
   MainWindow( const MainWindow& );
   MainWindow& operator=( const MainWindow& );

// ******* CONSTANTS *******
private:
   static const char* const TREE_TITLE;
   static const char* const DEFAULT_STATUS;
   static const char* const MENU_SYSTEM_LABEL;
   static const char* const MENU_TRACE_LABEL;
   static const char* const MENU_HELP_LABEL;
   static const char* const ACTION_SERVER_LABEL;
   static const char* const ACTION_CONNECT_LABEL;
   static const char* const ACTION_DISCONNECT_LABEL;
   static const char* const ACTION_RUN_LABEL;
   static const char* const ACTION_EXIT_LABEL;
   static const char* const ACTION_EXIT_KEY;

// ******* MEMBERS *******
private:
   QMenu*       const menu_system_;
   QMenu*       const menu_trace_;
   QMenu*       const menu_help_;
   QToolBar*    const toolbar_;
   QAction*     const action_server_;
   QAction*     const action_connect_;
   QAction*     const action_disconnect_;
   QAction*     const action_run_;
   QAction*     const action_exit_;
   QDockWidget* const dock_;
   Table*       const table_;
   Tree*        const tree_;
   Network*     const network_;
   Monitor*     const monitor_;

 // ******* METHODS *******
private:
   void customEvent     ( QEvent* );
   void showEvent       ( QShowEvent* );
   void save_settings   ();
   void restore_settings();
   void set_status      ( const QString& );
   void update_actions  ();
   void create_menu     ();
   void toolbar         ();
   void translate       ();
private slots:
   void server_slot     ();
   void connect_slot    ();
   void disconnect_slot ();
   void run_slot        ();
};

#endif // INCLUDED_MainWindow_h
