/********************************************************************
 * Copyright (C) 2006-2007 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Beesoft Tracer.
 *
 * Beesoft Tracer is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Beesoft Tracer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Beesoft Tracer; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Beesoft Tracer
 * FILE   : Message.h
 * DATE   : 01.07.2007
 *******************************************************************/
#ifndef INCLUDED_Message_h
#define INCLUDED_Message_h

/*------- incude files:
-------------------------------------------------------------------*/
#include <QObject>
#include <QString>

/*------- class declaration:
-------------------------------------------------------------------*/
class Message : public QObject
{
// ******* CONSTRUCTOR / DESTRUCTION *******
public:
   Message()
   : QObject(), type_( -1 ), id_( -1 ), text_( "" )
   {}
   Message( const qint32 in_type, const qint32 in_id )
   : QObject(), type_( in_type ), id_( in_id ), text_( "" )
   {}
   Message( const qint32 in_type, const qint32 in_id, const QString& in_text )
   : QObject(), type_( in_type ), id_( in_id ), text_( in_text )
   {}
   Message( const Message& rhs )
   {
      type_ = rhs.type_;
      id_   = rhs.id_;
      text_ = rhs.text_;
   }
   Message& operator=( const Message& rhs )
   {
      if( this != &rhs ) {
         type_ = rhs.type_;
         id_   = rhs.id_;
         text_ = rhs.text_;
      }
      return *this;
   }
// ******* MEMBERS *******
private:
   qint32  type_;
   qint32  id_;
   QString text_;

 // ******* METHODS *******
public:
   qint32         type() const { return type_; }
   qint32         id  () const { return id_;   }
   const QString& text() const { return text_; }
};

#endif // INCLUDED_Message_h
