/********************************************************************
 * Copyright (C) 2006-2007 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Beesoft Tracer.
 *
 * Beesoft Tracer is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Beesoft Tracer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Beesoft Tracer; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Beesoft Tracer
 * FILE   : MessageBase.cpp
 * DATE   : 01.07.2007
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "MessageBase.h"
#include <QApplication>
using namespace std;

/*------- local variables:
-------------------------------------------------------------------*/
MessageBase*   MessageBase::instance_ = 0;

//*******************************************************************
// MessageBase                                   CONSTRUCTOR private
//*******************************************************************
MessageBase::MessageBase() : QObject()
, is_running_ ( true )
{
   setParent( qApp );
   mutex_.set_name( "MessageBase" );
}
// end of MessageBase

//*******************************************************************
// ~MessageBase                                   DESTRUCTOR private
//*******************************************************************
MessageBase::~MessageBase()
{
   stop();
   mutex_.unlock();
}
// end of ~MessageBase

//*******************************************************************
// instance                                            PUBLIC static
//*******************************************************************
MessageBase* const MessageBase::instance()
{
   if( !instance_ ) {
      instance_ = new MessageBase;
   }
   return instance_;
}
// end of instance

//*******************************************************************
// add                                                        PUBLIC
//*******************************************************************
void MessageBase::add( const Message& in_msg )
{
   if( is_running_ ) {
      mutex_.lock();
      data_.push( in_msg );
      not_empty_.wake_up();
      mutex_.unlock();
   }
}
// end of add

//*******************************************************************
// get                                                        PUBLIC
//*******************************************************************
bool MessageBase::get( Message& out_msg )
{
   bool retval = false;

   if( is_running_ ) {
      mutex_.lock();
      while( is_running_ && data_.empty() ) {
         not_empty_.wait( &mutex_ );
      }
      if( is_running_ ) {
         out_msg = data_.front();
         data_.pop();
         retval = true;
      }
      mutex_.unlock();
   }
   
   return retval;
}
// end of get

//*******************************************************************
// stop                                                       PUBLIC
//*******************************************************************
void MessageBase:: stop()
{
   if( is_running_ ) {
      is_running_ = false;
   }
}
// end of stop
