/********************************************************************
 * Copyright (C) 2006-2007 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Beesoft Tracer.
 *
 * Beesoft Tracer is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Beesoft Tracer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Beesoft Tracer; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Beesoft Tracer
 * FILE   : Monitor.h
 * DATE   : 01.07.2007
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "Monitor.h"
#include "Table.h"
#include "Scopes.h"
#include "MessageBase.h"
#include "EventsController.h"
#include <string>
#include <QtDebug>
using namespace std;

//*******************************************************************
// Monitor                                               CONSTRUCTOR
//*******************************************************************
Monitor::Monitor( Table* const in_table )
: BTThread    ()
, QObject     ()
, table_      ( in_table )
, is_running_ ( false )
{
   EventsController::instance()->append( this, Event::SCOPES_SELECTED );
}
// end of Monitor

//*******************************************************************
// ~Monitor                                               DESTRUCTOR
//*******************************************************************
Monitor::~Monitor()
{
   stop();
   EventsController::instance()->remove( this );
}
// end of ~Monitor

//*******************************************************************
// customEvent                                     PRIVATE inherited
//*******************************************************************
void Monitor::customEvent( QEvent* const in_event )
{
   if( !is_running_ ) {
      Event* const event = dynamic_cast< Event* >( in_event );
      const qint32 type = static_cast<qint32>( event->type() );
      switch( type ) {
         case Event::SCOPES_SELECTED:
            start();
            break;
      }
   }
}
// end of customEvent

//*******************************************************************
// worker                                          PRIVATE inherited
//*******************************************************************
void Monitor::worker()
{
   is_running_ = true;
   
   Message msg;
   while( true ) {
      test_cancel();
      if( MessageBase::instance()->get( msg ) ) {
         string class_name;
         string method_name;
         if( Scopes::instance()->find( msg.id(), class_name, method_name ) ) {
            EventsController::instance()->send_event(
               Event::DISPLAY_MESSAGE, 
               msg.type(),
               class_name.c_str(),
               method_name.c_str(),
               msg.text() );
         }
      }
   }
}
// end of worker

//*******************************************************************
// stop                                                       PUBLIC
//*******************************************************************
void Monitor::stop()
{
   if( is_running_ ) {
      cancel();
      is_running_ = false;
   }
}
// end of stop
