/********************************************************************
 * Copyright (C) 2006-2007 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Beesoft Tracer.
 *
 * Beesoft Tracer is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Beesoft Tracer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Beesoft Tracer; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Beesoft Tracer
 * FILE   : Network.h
 * DATE   : 01.07.2007
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "Network.h"
#include "Scopes.h"
#include "Message.h"
#include "MessageBase.h"
#include "EventsController.h"
#include <Tracer.h>
#include <iostream>
using namespace std;

//*******************************************************************
// Network                                               CONSTRUCTOR
//*******************************************************************
Network::Network() : BTThread()
, addr_       ( "" )
, port_       ( -1 )
, socket_     (  0 )
, is_running_ ( false )
{}
// end of Network

//*******************************************************************
// ~Network                                              CONSTRUCTOR
//*******************************************************************
Network::~Network()
{
   disconnect();
}
// end of ~Network

//*******************************************************************
// clean_up                                                  PRIVATE
//*******************************************************************
void Network::clean_up()
{
   if( socket_ ) {
      socket_->close();
      delete socket_;
      socket_ = 0;
   }
}
// end of clean_up

//*******************************************************************
// connect_host                                               PUBLIC
//*******************************************************************
void Network::connect_host( const string& in_addr, const int in_port )
{
   if( !is_running_ ) {
      try {
         socket_ = new BTTcpSocket( in_addr, in_port );
         socket_->connect();
         EventsController::instance()->send_event( Event::CONNECTED );
         start();
      }
      catch( BTSocketException& in_exception ) {
         clean_up();
         cerr << "Error: " << in_exception.what() << endl;
      }
   }
}
// end of connect

//*******************************************************************
// disconnect                                                 PUBLIC
//*******************************************************************
void Network::disconnect()
{
   if( is_running_ ) {
      cancel();
      clean_up();
      EventsController::instance()->send_event( Event::DISCONNECTED );   
      is_running_ = false;
   }
}
// end of disconnect

//*******************************************************************
// worker                                                    PRIVATE
//*******************************************************************
void Network::worker()
{
   is_running_ = true;
   
   try {
      while( true ) {
         test_cancel();
         if( socket_->is_valid() ) {
            read_message();
         }
      }
   }
   catch( BTSocketException& in_exception ) {
      cerr << "Error: " << in_exception.what() << endl;
      clean_up();
      EventsController::instance()->send_event( Event::DISCONNECTED );  
      is_running_ = false;
   }
}
// end of worker

//*******************************************************************
// read_next
//*******************************************************************
bool Network::read_next( const int in_type ) const
{
   return !( ( Tracer::SCOPE_IN == in_type ) || ( Tracer::SCOPE_OUT == in_type ) );
}
// end of read_next

//*******************************************************************
// read_message                                              PRIVATE
//*******************************************************************
bool Network::read_message()
{
   static bool is_scopes_ready = false;
   bool retval = false;

   int type, id, n;
   string text;

   if(  socket_->read<int>( type ) ) {
      if( socket_->read<int>( id ) ) {
         if( read_next( type ) ) {
            if( socket_->read<int>( n ) ) {
               if( socket_->read( text, n ) ) {
                  if( Tracer::SCOPE_DEF == type ) {
                     retval = true;
                     /*
                     cerr << "======= SCOPE =======" << endl;
                     cerr << "id: " << id << endl;
                     cerr << "text: " << text << endl;
                     */
                     Scopes::instance()->add( id, text.c_str()  );
                  }
                  else {
                     retval = true;
                     if( !is_scopes_ready ) {
                        EventsController::instance()->send_event( Event::SCOPES_READY );
                        is_scopes_ready = true;
                     }
                     /*
                     cerr << "======= MESSAGE =======";
                     cerr << "type: " << type << endl;
                     cerr << "id: " << id << endl;
                     cerr << "text: " << text << endl;
                     */
                     MessageBase::instance()->add( Message( type, id, text.c_str() ) );
                  }
               }
            }
         }
         else {
            retval = true;
            if( !is_scopes_ready ) {
               EventsController::instance()->send_event( Event::SCOPES_READY );
               is_scopes_ready = true;
            }
            /*
            cerr << "======= IN/OUT =======";
            cerr << "type: " << type << endl;
            cerr << "id: " << id << endl;
            */
            MessageBase::instance()->add( Message( type, id ) );
         }
      }
   }
   return retval;
}
// end of read_message
