/********************************************************************
 * Copyright (C) 2006-2007 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Beesoft Tracer.
 *
 * Beesoft Tracer is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Beesoft Tracer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Beesoft Tracer; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Beesoft Tracer
 * FILE   : Scopes.cpp
 * DATE   : 01.07.2007
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "Scopes.h"
#include <ScopeDefs.h>
#include <QApplication>
using namespace std;

/*------- local variable:
-------------------------------------------------------------------*/
Scopes* Scopes::instance_ = 0;

//*******************************************************************
// Scopes                                        CONSTRUCTOR private
//*******************************************************************
Scopes::Scopes() : QObject()
{
   setParent( qApp );
   mutex_.set_name( "Scopes" );
}
// end of Scopes

//*******************************************************************
// ~Scopes                                        DESTRUCTOR private
//*******************************************************************
Scopes::~Scopes()
{}
// end of ~Scopes

//*******************************************************************
// instance                                            PUBLIC static
//*******************************************************************
Scopes* const Scopes::instance()
{
   if( !instance_ ) {
      instance_ = new Scopes;
   }
   return instance_;
}
// end of instance

//*******************************************************************
// add                                                        PUBLIC
//*******************************************************************
void Scopes::add( const int in_id, const string& in_text )
{
   mutex_.lock();

   const string::size_type idx = in_text.find( '|' );
   if( idx != string::npos ) {
      const string class_name = in_text.substr( 0, idx );
      const string method_name = in_text.substr( idx + 1 );

      ScopeInfo si( class_name, method_name );
      scopes_.add( in_id, si );
   }
   
   mutex_.unlock();
}
// end of add

//*******************************************************************
// first                                                      PUBLIC
//*******************************************************************
bool Scopes::first( int& out_id, ScopeInfo& out_scope )
{
   bool retval = false;
   InvScopeDefs::InvItemsMap::const_iterator it = scopes_.first();
   if( it != scopes_.end() ) {
      out_id = it->first;
      out_scope = it->second;
      retval = true;
   }
   return retval;
}
// end of first

//*******************************************************************
// first                                                      PUBLIC
//*******************************************************************
bool Scopes::find( const int in_id, string& in_class, string& in_method )
{
   bool retval = false;
   if( selected_.find( in_id ) != selected_.end() ) {
      ScopeInfo si;
      if( scopes_.scope( in_id, si ) ) {
         in_class = si.class_;
         in_method = si.method_;
         retval = true;
      }
   }
   return retval;
}
// end of find

//*******************************************************************
// next                                                       PUBLIC
//*******************************************************************
bool Scopes::next( int& out_id, ScopeInfo& out_scope )
{
   bool retval = false;
   InvScopeDefs::InvItemsMap::const_iterator it = scopes_.next();
   if( it != scopes_.end() ) {
      out_id = it->first;
      out_scope = it->second;
      retval = true;
   }
   return retval;
}
// end of next
