/********************************************************************
 * Copyright (C) 2006-2007 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Beesoft Tracer.
 *
 * Beesoft Tracer is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Beesoft Tracer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Beesoft Tracer; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Beesoft Tracer
 * FILE   : ServerEdit.cpp
 * DATE   : 02.07.2007
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "ServerEdit.h"
#include "Shared.h"
#include <QPixmap>
#include <QPushButton>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QLabel>
#include <QLineEdit>
#include <QGroupBox>
#include <QGridLayout>
#include <QIntValidator>

/*------- local constants:
---------------------------------------------------------*/
const char* const ServerEdit::TITLE        = QT_TR_NOOP( "Trace server data" );
const char* const ServerEdit::NAME_LABEL   = QT_TR_NOOP( "&Name:"            );
const char* const ServerEdit::ADDR_LABEL   = QT_TR_NOOP( "&Address:"         );
const char* const ServerEdit::PORT_LABEL   = QT_TR_NOOP( "&Port:"            );
const char* const ServerEdit::CANCEL_LABEL = QT_TR_NOOP( "&Cancel"           );
const char* const ServerEdit::ACCEPT_LABEL = QT_TR_NOOP( "Acce&pt"           );
const char* const ServerEdit::BOX_TITLE    = QT_TR_NOOP( "Server data"       );


//*******************************************************************
// ServerEdit                                            CONSTRUCTOR
//*******************************************************************
ServerEdit::ServerEdit( QWidget* const in_parent,
                        const QString& in_name,
                        const QString& in_addr,
                        const QString& in_port )
: QDialog   ( in_parent )
, name_edt_ ( new QLineEdit( in_name ) )
, addr_edt_ ( new QLineEdit( in_addr ) )
, port_edt_ ( new QLineEdit( in_port ) )
{
   setWindowTitle( tr( TITLE ) );
   port_edt_->setValidator( new QIntValidator( 1024, 100000, port_edt_ ) );

   QHBoxLayout* const main_layout = new QHBoxLayout( this );

   {  // Lewy panel z obrazkiem
      QVBoxLayout* const pic_layout = new QVBoxLayout;
      QLabel* const pic_label = new QLabel;
      pic_label->setPixmap( QPixmap( ":/img/servers.png" ) );
      pic_layout->setMargin( 20 );
      pic_layout->addWidget( pic_label );
      pic_layout->addStretch( 90 );
      main_layout->addLayout( pic_layout );
   }

   {  // panel z tablica
      QVBoxLayout* const in_layout = new QVBoxLayout;
      {  // nazwa serwera
         QHBoxLayout* const name_layout = new QHBoxLayout;
         QLabel* const name_label = new QLabel( tr( NAME_LABEL ) );
         name_label->setBuddy( name_edt_ );
         name_layout->addWidget( name_label );
         name_layout->addWidget( name_edt_ );
         in_layout->addLayout( name_layout );
      }
      {  // dane hosta
         QGroupBox* const gbox = new QGroupBox( tr( BOX_TITLE ) );
         QGridLayout* const gbox_layout = new QGridLayout( gbox );
         QLabel* const addr_label = new QLabel( tr( ADDR_LABEL ) );
         QLabel* const port_label = new QLabel( tr( PORT_LABEL ) );
         addr_label->setBuddy( addr_edt_ );
         port_label->setBuddy( port_edt_ );
         gbox_layout->addWidget( addr_label, 0, 0 );
         gbox_layout->addWidget( port_label, 1, 0 );
         gbox_layout->addWidget( addr_edt_ , 0, 1 );
         gbox_layout->addWidget( port_edt_ , 1, 1 );
         in_layout->addWidget( gbox );
      }
      in_layout->addStretch( 100 );
      main_layout->addLayout( in_layout );
   }

   {  // prawy panel z przyciskami
      QPushButton*  const cancel_btn = new QPushButton( tr( CANCEL_LABEL ) );
      QPushButton*  const ok_btn     = new QPushButton( tr( ACCEPT_LABEL ) );
   
      QVBoxLayout* const btn_layout = new QVBoxLayout;
      btn_layout->setSpacing( 2 );
      btn_layout->addStretch( 100 );
      btn_layout->addWidget( cancel_btn );
      btn_layout->addWidget( ok_btn     );
      main_layout->addLayout( btn_layout );

      connect( cancel_btn, SIGNAL( clicked() ), this, SLOT( reject() ) );
      connect( ok_btn    , SIGNAL( clicked() ), this, SLOT( ok()     ) );
   }
}
// end of ServerEdit

//*******************************************************************
// ok                                                   PRIVATE slot
//*******************************************************************
void ServerEdit::ok()
{
   if( name_edt_->text().isEmpty() ) {
      name_edt_->setFocus( Qt::OtherFocusReason );
      return;
   }
   if( addr_edt_->text().isEmpty() ) {
      addr_edt_->setText( Shared::default_srv_addr() );
      addr_edt_->setFocus( Qt::OtherFocusReason );
      return;
   }
   if( port_edt_->text().isEmpty() ) {
      port_edt_->setText( QString::number( Shared::default_srv_port() ) );
      port_edt_->setFocus( Qt::OtherFocusReason );
      return;
   }
   {
      qint32 pos = 0;
      QString port_text = port_edt_->text();
      if( port_edt_->validator()->validate( port_text, pos ) != QValidator::Acceptable ) {
         port_edt_->setFocus( Qt::OtherFocusReason );
         return;
      }
   }
   accept();
}
// end of ok

//*******************************************************************
// data                                                       PUBLIC
//*******************************************************************
void ServerEdit::data( QString& out_name, QString& out_addr, QString& out_port )
{
   out_name = name_edt_->text();
   out_addr = addr_edt_->text();
   out_port = port_edt_->text();
}
// end of data
