/********************************************************************
 * Copyright (C) 2006-2007 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Beesoft Tracer.
 *
 * Beesoft Tracer is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Beesoft Tracer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Beesoft Tracer; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Beesoft Tracer
 * FILE   : ServersDialog.cpp
 * DATE   : 02.07.2007
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "ServersDialog.h"
#include "ServersTable.h"
#include "Settings.h"
#include "ServerEdit.h"
#include "Shared.h"
#include <QPixmap>
#include <QPushButton>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QLabel>

/*------- local constants:
---------------------------------------------------------*/
const char* const ServersDialog::TITLE            = QT_TR_NOOP( "Trace servers" );
const char* const ServersDialog::ADD_BTN_LABEL    = QT_TR_NOOP( "&Add"          );
const char* const ServersDialog::EDIT_BTN_LABEL   = QT_TR_NOOP( "&Edit"         );
const char* const ServersDialog::DEL_BTN_LABEL    = QT_TR_NOOP( "&Remove"       );
const char* const ServersDialog::CANCEL_BTN_LABEL = QT_TR_NOOP( "&Cancel"       );
const char* const ServersDialog::OK_BTN_LABEL     = QT_TR_NOOP( "&Accept"       );
const char* const ServersDialog::SRV_NAME_DEFAULT = QT_TR_NOOP( "Local host"    );


//*******************************************************************
// ServersDialog                                         CONSTRUCTOR
//*******************************************************************
ServersDialog::ServersDialog( QWidget* const in_parent ) : QDialog( in_parent )
, table_      ( new ServersTable )
{
   setWindowTitle( tr( TITLE ) );

   QHBoxLayout* const main_layout = new QHBoxLayout( this );

   // Lewy panel z obrazkiem
   QVBoxLayout* const pic_layout = new QVBoxLayout;
   QLabel* const pic_label = new QLabel;
   pic_label->setPixmap( QPixmap( ":/img/servers.png" ) );
   pic_layout->setMargin( 20 );
   pic_layout->addWidget( pic_label );
   pic_layout->addStretch( 90 );
   main_layout->addLayout( pic_layout );

   // panel z tablica
   main_layout->addWidget( table_ );

   // prawy panel z przyciskami
   QPushButton*  const add_btn    = new QPushButton( tr( ADD_BTN_LABEL    ) );
   QPushButton*  const edit_btn   = new QPushButton( tr( EDIT_BTN_LABEL   ) );
   QPushButton*  const del_btn    = new QPushButton( tr( DEL_BTN_LABEL    ) );
   QPushButton*  const cancel_btn = new QPushButton( tr( CANCEL_BTN_LABEL ) );
   QPushButton*  const ok_btn     = new QPushButton( tr( OK_BTN_LABEL     ) );
   
   QVBoxLayout* const btn_layout = new QVBoxLayout;
   btn_layout->setSpacing( 2 );
   btn_layout->addStretch( 80 );
   btn_layout->addWidget( add_btn    );
   btn_layout->addWidget( edit_btn   );
   btn_layout->addWidget( del_btn    );
   btn_layout->addStretch( 20 );
   btn_layout->addWidget( cancel_btn );
   btn_layout->addWidget( ok_btn     );
   main_layout->addLayout( btn_layout );

   table_->add_data( Shared::default_srv_name(),
                     Shared::default_srv_addr(),
                     QString::number( Shared::default_srv_port() ) );
   restore_data();
   
   connect( add_btn   , SIGNAL( clicked() ), this, SLOT( add()    ) );
   connect( edit_btn  , SIGNAL( clicked() ), this, SLOT( edit()   ) );
   connect( del_btn   , SIGNAL( clicked() ), this, SLOT( del()    ) );
   connect( cancel_btn, SIGNAL( clicked() ), this, SLOT( reject() ) );
   connect( ok_btn    , SIGNAL( clicked() ), this, SLOT( ok()     ) );
}
// end of ServersDialog

//*******************************************************************
// add                                                  PRIVATE slot
//*******************************************************************
void ServersDialog::add()
{
   QString name, addr, port;

   ServerEdit* const se = new ServerEdit( this, name, addr, port );
   if( se ) {
      if( Accepted == se->exec() ) {
         se->data( name, addr, port );
         table_->add_data( name, addr, port );
      }
      delete se;
   }
}
// end of 

//*******************************************************************
// edit                                                  PRIVATE slot
//*******************************************************************
void ServersDialog::edit()
{
   const qint32 row = table_->currentRow();
   if( row > 0 ) {
      QString name = table_->item( row, 0 )->text();
      QString addr = table_->item( row, 1 )->text();
      QString port = table_->item( row, 2 )->text();
      ServerEdit* const se = new ServerEdit( this, name, addr, port );
      if( se ) {
         if( Accepted == se->exec() ) {
            se->data( name, addr, port );
            table_->set_data( row, name, addr, port );
         }
         delete se;
      }
   }
}
// end of edit

//*******************************************************************
// del                                                  PRIVATE slot
//*******************************************************************
void ServersDialog::del()
{
   QTableWidgetItem* const item = table_->item( table_->currentRow(), 0 );
   if( item ) {
      if( item->text() != Shared::default_srv_name() ) {
         table_->removeRow( item->row() );
      }
   }
}
// end of del

//*******************************************************************
// ok                                                   PRIVATE slot
//*******************************************************************
void ServersDialog::ok()
{
   
   const QTableWidgetItem* const item = table_->currentItem();
   if( item ) {
      save_data();
      {  // Zapamietanie danych serwera, ktory bedzie uzywany.
         const qint32 row = item->row();
         Shared::srv_name( table_->item( row, 0 )->text() );
         Shared::srv_addr( table_->item( row, 1 )->text() );
         Shared::srv_port( table_->item( row, 2 )->text() );
      }
      accept();
   }
}
// end of ok

//*******************************************************************
// save_data                                                 PRIVATE
//*******************************************************************
void ServersDialog::save_data()
{
   QStringList data;
   const qint32 n = table_->rowCount();
   for( qint32 row = 1; row < n; ++row ) {
      QString text = table_->item( row, 0 )->text();
      text += "|";
      text += table_->item( row, 1 )->text();
      text += "|";
      text += table_->item( row, 2 )->text();
      data << text;
   }
   
   Settings stt;
   stt.save( "ServersDialog/Data", data );
}
// end of save_data

//*******************************************************************
// restore_data                                              PRIVATE
//*******************************************************************
void ServersDialog::restore_data()
{
   Settings stt;
   QStringList data, items;
   QVariant vdata;

   if( stt.read( "ServersDialog/Data", vdata ) ) {
      data = vdata.toStringList();
      QStringList::const_iterator it = data.begin();
      const QStringList::const_iterator end = data.end();
      while( it != end ) {
         items = it->split( '|' );
         if( 3 == items.size() ) {
            const QString name = items[ 0 ];
            const QString addr = items[ 1 ];
            const QString port = items[ 2 ];
            if( -1 == table_->search_item( name, addr, port ) ) {
               table_->add_data( name, addr, port );
            }
         }
         ++it;
      }
   }
   
   // Podswietlenie aktualnie uzywanego serwera.
   const qint32 row = table_->search_item( Shared::srv_name(), Shared::srv_addr(), Shared::srv_port() );
   table_->set_current_item( ( row != -1 ) ? row : 0 );
}
// end of restore_data
