/********************************************************************
 * Copyright (C) 2006-2007 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Beesoft Tracer.
 *
 * Beesoft Tracer is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Beesoft Tracer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Beesoft Tracer; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Beesoft Tracer
 * FILE   : ServersTable.h
 * DATE   : 02.07.2007
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "ServersTable.h"
#include <QHeaderView>
#include <QtDebug>

/*------- local constants:
-------------------------------------------------------------------*/
const char* const ServersTable::NAME = QT_TR_NOOP( "name"    );
const char* const ServersTable::ADDR = QT_TR_NOOP( "address" );
const char* const ServersTable::PORT = QT_TR_NOOP( "port"    );

//*******************************************************************
// ServersTable                                          CONSTRUCTOR
//*******************************************************************
ServersTable::ServersTable( QWidget* const in_parent ) : QTableWidget( in_parent )
{
   setColumnCount( 3 );
   setEditTriggers( NoEditTriggers );
   setSelectionMode( SingleSelection );
   setSelectionBehavior( SelectRows );
   setAlternatingRowColors( true );

   verticalHeader()->hide();
   horizontalHeader()->setDefaultAlignment( Qt::AlignLeft );

   QStringList hdrtxt;
   hdrtxt << tr( NAME ) << tr( ADDR ) << tr( PORT );
   setHorizontalHeaderLabels ( hdrtxt );
   resize_columns();
}
// end of ServersTable

//*******************************************************************
// resizeEvent                                     PRIVATE inherited
//*******************************************************************
void ServersTable::resizeEvent( QResizeEvent* const in_event )
{
   QTableWidget::resizeEvent( in_event );
   resize_columns();
}
// end of resizeEvent

//*******************************************************************
// search_item                                                PUBLIC
//*******************************************************************
qint32 ServersTable::search_item( const QString& in_name, const QString& in_addr, const QString& in_port )
{
   qint32 retval = -1;
   
   const qint32 n = rowCount();
   for( qint32 row = 0; row < n; ++row ) {
      if( in_name == item( row, 0 )->text() ) {
         if( in_addr == item( row, 1 )->text() ) {
            if( in_port == item( row, 2 )->text() ) {
               retval = row;
               break;
            }
         }
      }
   }
   return retval;
}
// end of search_item

//*******************************************************************
// add_data                                                   PUBLIC
//*******************************************************************
void ServersTable::add_data( const QString& in_name, const QString& in_addr, const QString& in_port )
{
   const qint32 row = rowCount();
   insertRow( row );

   setItem( row, 0, new QTableWidgetItem( in_name ) );
   setItem( row, 1, new QTableWidgetItem( in_addr ) );
   setItem( row, 2, new QTableWidgetItem( in_port ) );

   setRowHeight( row, fontMetrics().height() + 2 );
   resize_columns();
   set_current_item( row );
}
// end of add_data

//*******************************************************************
// set_data                                                   PUBLIC
//*******************************************************************
void ServersTable::set_data( const qint32 in_row, const QString& in_name, const QString& in_addr, const QString& in_port )
{
   if( ( in_row >= 0 ) && ( in_row < rowCount() ) ) {
      item( in_row, 0 )->setText( in_name );
      item( in_row, 1 )->setText( in_addr );
      item( in_row, 2 )->setText( in_port );

      resize_columns();
      set_current_item( in_row );
   }
}
   // end of set_data

//*******************************************************************
// set_current_item                                           PUBLIC
//*******************************************************************
void ServersTable::set_current_item( const qint32 in_row )
{
   QTableWidgetItem* const itm = item( in_row, 0 );
   if( itm ) {
      scrollToItem( itm );
      setCurrentItem( itm );
   }
}
// end of set_current_item

//*******************************************************************
// resize_columns                                            PRIVATE
//*******************************************************************
void ServersTable::resize_columns()
{
   resizeColumnsToContents();
   
   const qint32 view_width = viewport()->width();
   const qint32 total_width = total_columns_width();
   if( total_width < view_width ) {
      setColumnWidth( 2, view_width - ( total_width - columnWidth( 2 ) ) );
   }
}
// end of resize_columns

//*******************************************************************
// total_columns_width                                       PRIVATE
//*******************************************************************
qint32 ServersTable::total_columns_width() const
{
   qint32 retval = 0;
   const qint32 n = columnCount();

   for( qint32 i = 0; i < n; ++i ) {
      retval += columnWidth( i );
   }
   return retval;
}
// end of total_columns_width
