/********************************************************************
 * Copyright (C) 2006-2007 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Beesoft Tracer.
 *
 * Beesoft Tracer is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Beesoft Tracer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Beesoft Tracer; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Beesoft Tracer
 * FILE   : Shared.cpp
 * DATE   : 29.06.2007
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "Shared.h"
#include <QApplication>
#include <QDesktopWidget>

/*------- local constants:
-------------------------------------------------------------------*/
const char* const Shared::ORGANISATION             = "beesoft.org";
const char* const Shared::PROGRAM_NAME             = "TraceView";
const char* const Shared::PROGRAM_SUBNAME          = QT_TR_NOOP( "gui tracer client" );
const char* const Shared::VERSION                  = "0.02";
const char* const Shared::DEFAULT_SRV_NAME         = QT_TR_NOOP( "Local host" );
const char* const Shared::DEFAULT_SRV_ADDR         = "127.0.0.1";
qint32            Shared::DEFAULT_SRV_PORT         = 6789;

/*------- local variables:
-------------------------------------------------------------------*/
QString Shared::buffer_   = "";
QString Shared::srv_name_ = QString( DEFAULT_SRV_NAME );
QString Shared::srv_addr_ = QString( DEFAULT_SRV_ADDR );
QString Shared::srv_port_ = QString( DEFAULT_SRV_PORT );


//*******************************************************************
// program_name
//*******************************************************************
const QString& Shared::full_program_name()
{
   buffer_  = PROGRAM_NAME;
   buffer_ += " - ";
   buffer_ += tr( PROGRAM_SUBNAME );
   buffer_ += ", ver. ";
   buffer_ += VERSION;
   
   return buffer_;
}
// end of full_program_name

//*******************************************************************
// program_name
//*******************************************************************
const QString& Shared::program_name()
{
   return ( buffer_ = PROGRAM_NAME );
}
// end of program_name

//*******************************************************************
// organization
//*******************************************************************
const QString& Shared::organisation()
{
   return ( buffer_ = ORGANISATION );
}
// end of organization

//*******************************************************************
// default_srv_name
//*******************************************************************
const QString Shared::default_srv_name()
{
   return tr( DEFAULT_SRV_NAME );
}
// end of default_srv_name

//*******************************************************************
// default_srv_addr
//*******************************************************************
const QString Shared::default_srv_addr()
{
   return QString( DEFAULT_SRV_ADDR );
}
// end of default_srv_addr

//*******************************************************************
// default_srv_port
//*******************************************************************
qint32 Shared::default_srv_port()
{
   return DEFAULT_SRV_PORT;
}
// end of default_srv_port

//*******************************************************************
// resize
//*******************************************************************
void Shared::resize( QWidget* const in_widget, const qint32 in_width, const qint32 in_height )
{
   const qint32 screen_width   = qApp->desktop()->availableGeometry().width();
   const qint32 screen_height  = qApp->desktop()->availableGeometry().height();
   const qreal  width_percent  = static_cast<qreal>( in_width ) / 100.0;
   const qreal  height_percent = static_cast<qreal>( in_height ) / 100.0;    

   const qint32 dx = static_cast<qint32>( screen_width  * width_percent );
   const qint32 dy = static_cast<qint32>( screen_height * height_percent );
   const qint32 x  = ( screen_width  - dx ) >> 1;
   const qint32 y  = ( screen_height - dy ) >>  1;

   in_widget->move( x, y );
   in_widget->resize( dy, dx );
}
// end of resize

//*******************************************************************
// srv_name (v.get)
//*******************************************************************
const QString& Shared::srv_name()
{
   return srv_name_;
}
// end of srv_name

//*******************************************************************
// srv_addr (v.get)
//*******************************************************************
const QString& Shared::srv_addr()
{
   return srv_addr_;
}
// end of srv_addr

//*******************************************************************
// srv_port (v.get)
//*******************************************************************
const QString& Shared::srv_port()
{
   return srv_port_;
}
// end of srv_port

//*******************************************************************
// srv_name (v.set)
//*******************************************************************
void Shared::srv_name( const QString& in_name )
{
   srv_name_ = in_name;
}
// end of srv_name

//*******************************************************************
// srv_addr (v.set)
//*******************************************************************
void Shared::srv_addr( const QString& in_addr )
{
   srv_addr_ = in_addr;
}
// end of srv_addr

//*******************************************************************
// srv_port (v.set)
//*******************************************************************
void Shared::srv_port( const QString& in_port )
{
   srv_port_ = in_port;
}
// end of srv_port

//*******************************************************************
// idle
//*******************************************************************
void Shared::idle()
{
   QApplication::processEvents( QEventLoop::AllEvents );
}
// end of idle
